@@ -20,8 +20,22 @@ function get_column_headers( $screen ) {
 
 	static $column_headers = array();
 
-	if ( ! isset( $column_headers[ $screen->id ] ) )
+	if ( ! isset( $column_headers[ $screen->id ] ) ) {
+
+		/**
+		 * Filter the column headers for a list table on a specific screen.
+		 *
+		 * The dynamic portion of the hook name, $screen->id, refers to the
+		 * ID of a specific screen. For example, the screen ID for the Posts
+		 * list table is edit-post, so the filter for that screen would be
+		 * manage_edit-post_columns.
+		 *
+		 * @since 3.0.0
+		 *
+		 * @param array $columns An array of column headers. Default empty.
+		 */
 		$column_headers[ $screen->id ] = apply_filters( 'manage_' . $screen->id . '_columns', array() );
+	}
 
 	return $column_headers[ $screen->id ];
 }
@@ -96,14 +110,33 @@ function get_hidden_meta_boxes( $screen ) {
 	if ( $use_defaults ) {
 		$hidden = array();
 		if ( 'post' == $screen->base ) {
-			if ( 'post' == $screen->post_type || 'page' == $screen->post_type )
+			if ( 'post' == $screen->post_type || 'page' == $screen->post_type || 'attachment' == $screen->post_type )
 				$hidden = array('slugdiv', 'trackbacksdiv', 'postcustom', 'postexcerpt', 'commentstatusdiv', 'commentsdiv', 'authordiv', 'revisionsdiv');
 			else
 				$hidden = array( 'slugdiv' );
 		}
+
+		/**
+		 * Filter the default list of hidden meta boxes.
+		 *
+		 * @since 3.1.0
+		 *
+		 * @param array     $hidden An array of meta boxes hidden by default.
+		 * @param WP_Screen $screen WP_Screen object of the current screen.
+		 */
 		$hidden = apply_filters( 'default_hidden_meta_boxes', $hidden, $screen );
 	}
 
+	/**
+	 * Filter the list of hidden meta boxes.
+	 *
+	 * @since 3.3.0
+	 *
+	 * @param array     $hidden       An array of hidden meta boxes.
+	 * @param WP_Screen $screen       WP_Screen object of the current screen.
+	 * @param bool      $use_defaults Whether to show the default meta boxes.
+	 *                                Default true.
+	 */
 	return apply_filters( 'hidden_meta_boxes', $hidden, $screen, $use_defaults );
 }
 
@@ -124,52 +157,6 @@ function add_screen_option( $option, $args = array() ) {
 	$current_screen->add_option( $option, $args );
 }
 
-/**
- * Displays a screen icon.
- *
- * @uses get_screen_icon()
- * @since 2.7.0
- *
- * @param string|WP_Screen $screen Optional. Accepts a screen object (and defaults to the current screen object)
- * 	which it uses to determine an icon HTML ID. Or, if a string is provided, it is used to form the icon HTML ID.
- */
-function screen_icon( $screen = '' ) {
-	echo get_screen_icon( $screen );
-}
-
-/**
- * Gets a screen icon.
- *
- * @since 3.2.0
- *
- * @param string|WP_Screen $screen Optional. Accepts a screen object (and defaults to the current screen object)
- * 	which it uses to determine an icon HTML ID. Or, if a string is provided, it is used to form the icon HTML ID.
- * @return string HTML for the screen icon.
- */
-function get_screen_icon( $screen = '' ) {
-	if ( empty( $screen ) )
-		$screen = get_current_screen();
-	elseif ( is_string( $screen ) )
-		$icon_id = $screen;
-
-	$class = 'icon32';
-
-	if ( empty( $icon_id ) ) {
-		if ( ! empty( $screen->parent_base ) )
-			$icon_id = $screen->parent_base;
-		else
-			$icon_id = $screen->base;
-
-		if ( 'page' == $screen->post_type )
-			$icon_id = 'edit-pages';
-
-		if ( $screen->post_type )
-			$class .= ' ' . sanitize_html_class( 'icon32-posts-' . $screen->post_type );
-	}
-
-	return '<div id="icon-' . esc_attr( $icon_id ) . '" class="' . $class . '"><br /></div>';
-}
-
 /**
  * Get the current screen object
  *
@@ -243,10 +230,22 @@ final class WP_Screen {
 	 */
 	public $id;
 
+	/**
+	 * Which admin the screen is in. network | user | site | false
+	 *
+	 * @since 3.5.0
+	 * @var string
+	 * @access protected
+	 */
+	protected $in_admin;
+
 	/**
 	 * Whether the screen is in the network admin.
 	 *
+	 * Deprecated. Use in_admin() instead.
+	 *
 	 * @since 3.3.0
+	 * @deprecated 3.5.0
 	 * @var bool
 	 * @access public
 	 */
@@ -255,7 +254,10 @@ final class WP_Screen {
 	/**
 	 * Whether the screen is in the user admin.
 	 *
+	 * Deprecated. Use in_admin() instead.
+	 *
 	 * @since 3.3.0
+	 * @deprecated 3.5.0
 	 * @var bool
 	 * @access public
 	 */
@@ -304,20 +306,20 @@ final class WP_Screen {
 
 	/**
 	 * The help tab data associated with the screen, if any.
- 	 *
- 	 * @since 3.3.0
- 	 * @var array
- 	 * @access private
- 	 */
+	 *
+	 * @since 3.3.0
+	 * @var array
+	 * @access private
+	 */
 	private $_help_tabs = array();
 
- 	/**
+	/**
 	 * The help sidebar data associated with screen, if any.
 	 *
 	 * @since 3.3.0
 	 * @var string
 	 * @access private
- 	 */
+	 */
 	private $_help_sidebar = '';
 
 	/**
@@ -361,23 +363,23 @@ final class WP_Screen {
 	 */
 	private $_screen_settings;
 
- 	/**
+	/**
 	 * Fetches a screen object.
- 	 *
- 	 * @since 3.3.0
+	 *
+	 * @since 3.3.0
 	 * @access public
- 	 *
+	 *
 	 * @param string $hook_name Optional. The hook name (also known as the hook suffix) used to determine the screen.
 	 * 	Defaults to the current $hook_suffix global.
 	 * @return WP_Screen Screen object.
- 	 */
+	 */
 	public static function get( $hook_name = '' ) {
 
 		if ( is_a( $hook_name, 'WP_Screen' ) )
 			return $hook_name;
 
 		$post_type = $taxonomy = null;
-		$is_network = $is_user = false;
+		$in_admin = false;
 		$action = '';
 
 		if ( $hook_name )
@@ -402,30 +404,39 @@ final class WP_Screen {
 		if ( ! $post_type && $hook_name ) {
 			if ( '-network' == substr( $id, -8 ) ) {
 				$id = substr( $id, 0, -8 );
-				$is_network = true;
+				$in_admin = 'network';
 			} elseif ( '-user' == substr( $id, -5 ) ) {
 				$id = substr( $id, 0, -5 );
-				$is_user = true;
+				$in_admin = 'user';
 			}
 
 			$id = sanitize_key( $id );
 			if ( 'edit-comments' != $id && 'edit-tags' != $id && 'edit-' == substr( $id, 0, 5 ) ) {
 				$maybe = substr( $id, 5 );
 				if ( taxonomy_exists( $maybe ) ) {
- 					$id = 'edit-tags';
+					$id = 'edit-tags';
 					$taxonomy = $maybe;
 				} elseif ( post_type_exists( $maybe ) ) {
 					$id = 'edit';
 					$post_type = $maybe;
 				}
- 			}
+			}
+
+			if ( ! $in_admin )
+				$in_admin = 'site';
 		} else {
-			$is_network = is_network_admin();
-			$is_user = is_user_admin();
+			if ( defined( 'WP_NETWORK_ADMIN' ) && WP_NETWORK_ADMIN )
+				$in_admin = 'network';
+			elseif ( defined( 'WP_USER_ADMIN' ) && WP_USER_ADMIN )
+				$in_admin = 'user';
+			else
+				$in_admin = 'site';
 		}
 
 		if ( 'index' == $id )
 			$id = 'dashboard';
+		elseif ( 'front' == $id )
+			$in_admin = false;
 
 		$base = $id;
 
@@ -438,7 +449,9 @@ final class WP_Screen {
 
 			switch ( $base ) {
 				case 'post' :
-					if ( isset( $_GET['post'] ) )
+					if ( isset( $_GET['post'] ) && isset( $_POST['post_ID'] ) && (int) $_GET['post'] !== (int) $_POST['post_ID'] )
+						wp_die( __( 'A post ID mismatch has been detected.' ), __( 'Sorry, you are not allowed to edit this item.' ), 400 );
+					elseif ( isset( $_GET['post'] ) )
 						$post_id = (int) $_GET['post'];
 					elseif ( isset( $_POST['post_ID'] ) )
 						$post_id = (int) $_POST['post_ID'];
@@ -456,7 +469,7 @@ final class WP_Screen {
 						$post_type = 'post';
 					break;
 			}
- 		}
+		}
 
 		switch ( $base ) {
 			case 'post' :
@@ -472,17 +485,24 @@ final class WP_Screen {
 			case 'edit-tags' :
 				if ( null === $taxonomy )
 					$taxonomy = 'post_tag';
+				// The edit-tags ID does not contain the post type. Look for it in the request.
+				if ( null === $post_type ) {
+					$post_type = 'post';
+					if ( isset( $_REQUEST['post_type'] ) && post_type_exists( $_REQUEST['post_type'] ) )
+						$post_type = $_REQUEST['post_type'];
+				}
+
 				$id = 'edit-' . $taxonomy;
 				break;
 		}
 
-		if ( $is_network ) {
+		if ( 'network' == $in_admin ) {
 			$id   .= '-network';
 			$base .= '-network';
-		} elseif ( $is_user ) {
+		} elseif ( 'user' == $in_admin ) {
 			$id   .= '-user';
 			$base .= '-user';
- 		}
+		}
 
 		if ( isset( self::$_registry[ $id ] ) ) {
 			$screen = self::$_registry[ $id ];
@@ -497,13 +517,14 @@ final class WP_Screen {
 		$screen->action     = $action;
 		$screen->post_type  = (string) $post_type;
 		$screen->taxonomy   = (string) $taxonomy;
-		$screen->is_user    = $is_user;
-		$screen->is_network = $is_network;
+		$screen->is_user    = ( 'user' == $in_admin );
+		$screen->is_network = ( 'network' == $in_admin );
+		$screen->in_admin   = $in_admin;
 
 		self::$_registry[ $id ] = $screen;
 
 		return $screen;
- 	}
+	}
 
 	/**
 	 * Makes the screen object the current screen.
@@ -516,6 +537,14 @@ final class WP_Screen {
 		$current_screen = $this;
 		$taxnow = $this->taxonomy;
 		$typenow = $this->post_type;
+
+		/**
+		 * Fires after the current screen has been set.
+		 *
+		 * @since 3.0.0
+		 *
+		 * @param WP_Screen $current_screen Current WP_Screen object.
+		 */
 		do_action( 'current_screen', $current_screen );
 	}
 
@@ -527,6 +556,23 @@ final class WP_Screen {
 	 */
 	private function __construct() {}
 
+	/**
+	 * Indicates whether the screen is in a particular admin
+	 *
+	 * @since 3.5.0
+	 *
+	 * @param string $admin The admin to check against (network | user | site).
+	 * If empty any of the three admins will result in true.
+	 * @return boolean True if the screen is in the indicated admin, false otherwise.
+	 *
+	 */
+	public function in_admin( $admin = null ) {
+		if ( empty( $admin ) )
+			return (bool) $this->in_admin;
+
+		return ( $admin == $this->in_admin );
+	}
+
 	/**
 	 * Sets the old string-based contextual help for the screen.
 	 *
@@ -568,6 +614,37 @@ final class WP_Screen {
 		$this->_options[ $option ] = $args;
 	}
 
+	/**
+	 * Remove an option from the screen.
+	 *
+	 * @since 3.8.0
+	 *
+	 * @param string $option Option ID.
+	 */
+	public function remove_option( $option ) {
+		unset( $this->_options[ $option ] );
+	}
+
+	/**
+	 * Remove all options from the screen.
+	 *
+	 * @since 3.8.0
+	 */
+	public function remove_options() {
+		$this->_options = array();
+	}
+
+	/**
+	 * Get the options registered for the screen.
+	 *
+	 * @since 3.8.0
+	 *
+	 * @return array Options with arguments.
+	 */
+	public function get_options() {
+		return $this->_options;
+	}
+
 	/**
 	 * Gets the arguments for an option for the screen.
 	 *
@@ -713,14 +790,46 @@ final class WP_Screen {
 	 */
 	public function render_screen_meta() {
 
-		// Call old contextual_help_list filter.
+		/**
+		 * Filter the legacy contextual help list.
+		 *
+		 * @since 2.7.0
+		 * @deprecated 3.3.0 Use get_current_screen()->add_help_tab() or
+		 *                   get_current_screen()->remove_help_tab() instead.
+		 *
+		 * @param array     $old_compat_help Old contextual help.
+		 * @param WP_Screen $this            Current WP_Screen instance.
+		 */
 		self::$_old_compat_help = apply_filters( 'contextual_help_list', self::$_old_compat_help, $this );
 
 		$old_help = isset( self::$_old_compat_help[ $this->id ] ) ? self::$_old_compat_help[ $this->id ] : '';
+
+		/**
+		 * Filter the legacy contextual help text.
+		 *
+		 * @since 2.7.0
+		 * @deprecated 3.3.0 Use get_current_screen()->add_help_tab() or
+		 *                   get_current_screen()->remove_help_tab() instead.
+		 *
+		 * @param string    $old_help  Help text that appears on the screen.
+		 * @param string    $screen_id Screen ID.
+		 * @param WP_Screen $this      Current WP_Screen instance.
+		 *
+		 */
 		$old_help = apply_filters( 'contextual_help', $old_help, $this->id, $this );
 
 		// Default help only if there is no old-style block of text and no new-style help tabs.
 		if ( empty( $old_help ) && ! $this->get_help_tabs() ) {
+
+			/**
+			 * Filter the default legacy contextual help text.
+			 *
+			 * @since 2.8.0
+			 * @deprecated 3.3.0 Use get_current_screen()->add_help_tab() or
+			 *                   get_current_screen()->remove_help_tab() instead.
+			 *
+			 * @param string $old_help_default Default contextual help text.
+			 */
 			$default_help = apply_filters( 'default_contextual_help', '' );
 			if ( $default_help )
 				$old_help = '<p>' . $default_help . '</p>';
@@ -744,7 +853,7 @@ final class WP_Screen {
 		?>
 		<div id="screen-meta" class="metabox-prefs">
 
-			<div id="contextual-help-wrap" class="<?php echo esc_attr( $help_class ); ?>">
+			<div id="contextual-help-wrap" class="<?php echo esc_attr( $help_class ); ?>" tabindex="-1" aria-label="<?php esc_attr_e('Contextual Help Tab'); ?>">
 				<div id="contextual-help-back"></div>
 				<div id="contextual-help-columns">
 					<div class="contextual-help-tabs">
@@ -757,7 +866,7 @@ final class WP_Screen {
 							?>
 
 							<li id="<?php echo esc_attr( $link_id ); ?>"<?php echo $class; ?>>
-								<a href="<?php echo esc_url( "#$panel_id" ); ?>">
+								<a href="<?php echo esc_url( "#$panel_id" ); ?>" aria-controls="<?php echo esc_attr( $panel_id ); ?>">
 									<?php echo esc_html( $tab['title'] ); ?>
 								</a>
 							</li>
@@ -801,7 +910,18 @@ final class WP_Screen {
 		<?php
 		// Setup layout columns
 
-		// Back compat for plugins using the filter instead of add_screen_option()
+		/**
+		 * Filter the array of screen layout columns.
+		 *
+		 * This hook provides back-compat for plugins using the back-compat
+		 * filter instead of add_screen_option().
+		 *
+		 * @since 2.8.0
+		 *
+		 * @param array     $empty_columns Empty array.
+		 * @param string    $screen_id     Screen ID.
+		 * @param WP_Screen $this          Current WP_Screen instance.
+		 */
 		$columns = apply_filters( 'screen_layout_columns', array(), $this->id, $this );
 
 		if ( ! empty( $columns ) && isset( $columns[ $this->id ] ) )
@@ -827,12 +947,12 @@ final class WP_Screen {
 		<div id="screen-meta-links">
 		<?php if ( $this->get_help_tabs() ) : ?>
 			<div id="contextual-help-link-wrap" class="hide-if-no-js screen-meta-toggle">
-			<a href="#contextual-help-wrap" id="contextual-help-link" class="show-settings"><?php _e( 'Help' ); ?></a>
+			<a href="#contextual-help-wrap" id="contextual-help-link" class="show-settings" aria-controls="contextual-help-wrap" aria-expanded="false"><?php _e( 'Help' ); ?></a>
 			</div>
 		<?php endif;
 		if ( $this->show_screen_options() ) : ?>
 			<div id="screen-options-link-wrap" class="hide-if-no-js screen-meta-toggle">
-			<a href="#screen-options-wrap" id="show-settings-link" class="show-settings"><?php _e( 'Screen Options' ); ?></a>
+			<a href="#screen-options-wrap" id="show-settings-link" class="show-settings" aria-controls="screen-options-wrap" aria-expanded="false"><?php _e( 'Screen Options' ); ?></a>
 			</div>
 		<?php endif; ?>
 		</div>
@@ -849,17 +969,41 @@ final class WP_Screen {
 
 		$show_screen = ! empty( $wp_meta_boxes[ $this->id ] ) || $columns || $this->get_option( 'per_page' );
 
-		$this->_screen_settings = apply_filters( 'screen_settings', '', $this );
-
 		switch ( $this->id ) {
 			case 'widgets':
-				$this->_screen_settings = '<p><a id="access-on" href="widgets.php?widgets-access=on">' . __('Enable accessibility mode') . '</a><a id="access-off" href="widgets.php?widgets-access=off">' . __('Disable accessibility mode') . "</a></p>\n";
+				$nonce = wp_create_nonce( 'widgets-access' );
+				$this->_screen_settings = '<p><a id="access-on" href="widgets.php?widgets-access=on&_wpnonce=' . urlencode( $nonce ) . '">' . __('Enable accessibility mode') . '</a><a id="access-off" href="widgets.php?widgets-access=off&_wpnonce=' . urlencode( $nonce ) . '">' . __('Disable accessibility mode') . "</a></p>\n";
+				break;
+			default:
+				$this->_screen_settings = '';
 				break;
 		}
 
+		/**
+		 * Filter the screen settings text displayed in the Screen Options tab.
+		 *
+		 * This filter is currently only used on the Widgets screen to enable
+		 * accessibility mode.
+		 *
+		 * @since 3.0.0
+		 *
+		 * @param string    $screen_settings Screen settings.
+		 * @param WP_Screen $this            WP_Screen object.
+		 */
+		$this->_screen_settings = apply_filters( 'screen_settings', $this->_screen_settings, $this );
+
 		if ( $this->_screen_settings || $this->_options )
 			$show_screen = true;
 
+		/**
+		 * Filter whether to show the Screen Options tab.
+		 *
+		 * @since 3.2.0
+		 *
+		 * @param bool      $show_screen Whether to show Screen Options tab.
+		 *                               Default true.
+		 * @param WP_Screen $this        Current WP_Screen instance.
+		 */
 		$this->_show_screen_options = apply_filters( 'screen_options_show_screen', $show_screen, $this );
 		return $this->_show_screen_options;
 	}
@@ -874,9 +1018,10 @@ final class WP_Screen {
 
 		$columns = get_column_headers( $this );
 		$hidden  = get_hidden_columns( $this );
+		$post    = get_post();
 
 		?>
-		<div id="screen-options-wrap" class="hidden">
+		<div id="screen-options-wrap" class="hidden" tabindex="-1" aria-label="<?php esc_attr_e('Screen Options Tab'); ?>">
 		<form id="adv-settings" action="" method="post">
 		<?php if ( isset( $wp_meta_boxes[ $this->id ] ) || $this->get_option( 'per_page' ) || ( $columns && empty( $columns['_title'] ) ) ) : ?>
 			<h5><?php _e( 'Show on screen' ); ?></h5>
@@ -888,7 +1033,7 @@ final class WP_Screen {
 				<?php
 					meta_box_prefs( $this );
 
-					if ( 'dashboard' === $this->id && current_user_can( 'edit_theme_options' ) ) {
+					if ( 'dashboard' === $this->id && has_action( 'welcome_panel' ) && current_user_can( 'edit_theme_options' ) ) {
 						if ( isset( $_GET['welcome'] ) ) {
 							$welcome_checked = empty( $_GET['welcome'] ) ? 0 : 1;
 							update_user_meta( get_current_user_id(), 'show_welcome_panel', $welcome_checked );
@@ -996,16 +1141,22 @@ final class WP_Screen {
 
 		if ( 'edit_comments_per_page' == $option ) {
 			$comment_status = isset( $_REQUEST['comment_status'] ) ? $_REQUEST['comment_status'] : 'all';
+
+			/** This filter is documented in wp-admin/includes/class-wp-comments-list-table.php */
 			$per_page = apply_filters( 'comments_per_page', $per_page, $comment_status );
 		} elseif ( 'categories_per_page' == $option ) {
+			/** This filter is documented in wp-admin/includes/class-wp-terms-list-table.php */
 			$per_page = apply_filters( 'edit_categories_per_page', $per_page );
 		} else {
+			/** This filter is documented in wp-admin/includes/class-wp-list-table.php */
 			$per_page = apply_filters( $option, $per_page );
 		}
 
 		// Back compat
-		if ( isset( $this->post_type ) )
+		if ( isset( $this->post_type ) ) {
+			/** This filter is documented in wp-admin/includes/class-wp-posts-list-table.php */
 			$per_page = apply_filters( 'edit_posts_per_page', $per_page, $this->post_type );
+		}
 
 		?>
 		<div class="screen-options">
